import * as Web3 from 'web3';
import VolumeSubscriptionWrapper from '../wrappers/volume_subscription_wrapper';
import ExecutorWrapper from '../wrappers/executor_wrapper';
import TokenWrapper from '../wrappers/token_wrapper';
import { TxData, TxHash, Bytes32, Address, AddressBook, Subscription } from '@8xprotocol/types';
/**
 * Subscriptions
 *
 * @comment The subscriptions api provides all the functionality for consumers to subscribe to 8x. A user must first subscribe to a subscription,
 * and then they must activate it. Payment is only taken on activation. First time 8x users are required to give pre-authorisation before
 * they can subscribe however.
 *
 * @path eightEx.subscriptions
 *
*/
export default class SubscriptionsAPI {
    private web3;
    private addressBook;
    private volumeSubscriptionWrapper;
    private executorWrapper;
    private tokenWrapper;
    constructor(web3: Web3, addressBook: AddressBook, volumeSubscriptionWrapper: VolumeSubscriptionWrapper, executorWrapper: ExecutorWrapper, tokenWrapper: TokenWrapper);
    /**
     * Check pre-authorisation
     *
     * In order for 8x to take tokens directly from a user's wallet, it needs authorisation from a user
     * to do so. This permission is given through the ERC20 approve function. Through this
     * functionality you can check whether a user has given approval to the Transfer Proxy (contract
     * that can take tokens from a user directly).
     *
     * @param owner     The user to check whether allowance has been given or not.
     *
     * @response
     * ```
     * [ true ]
     * ```
     *
     * @returns         A boolean of whether allowance has been given or not.
     * @priority        1
     *
    */
    hasGivenAuthorisation(owner: Address): Promise<boolean>;
    /**
     * Give pre-authorisation
     *
     * Grant 8x the pre-authorisation to take tokens from a user's wallet directly. The
     * pre-authorisation is given to the Transfer Proxy contract.
     *
     * @param txData    Provide signer, gas and gasPrice information (optional).
     * @priority        2
     *
    */
    giveAuthorisation(txData?: TxData): Promise<TxHash>;
    /**
     * Subscribe
     *
     * Subscribing links the user to the subscription plan. It does not take payment from the user.
     * Payment is only taken when the subscription is activated. This is useful in scenarios
     * where you might want to get a user's commitment and then charge them after a certain duration or amount of usage.
     *
     * @param planHash      Unique identifying hash of the plan.
     * @param metaData      Any extra JSON data you'd like to pass/store on-chain (optional).
     * @param txData        Provide signer, gas and gasPrice information (optional).
     *
     * @returns             Unique subscription hash.
     * @priority            4
     *
    */
    subscribe(planHash: Bytes32, metaData: JSON | null, txData?: TxData): Promise<Bytes32>;
    /**
     * Activate
     *
     * Makes first payment. This function can only be called once a user has subscribed to a subscription plan.
     * Provided that a user has subscribed and given authorisation to 8x, the activate function can be called
     * by any user. We're still exploring the best way to lock this down while still providing enough flexibility.
     *
     *
     * @param subscriptionHash    Unique subscription hash returned upon subscribing.
     * @param txData              Provide signer, gas and gasPrice information (optional).
     *
     * @returns                   Hash of the transaction upon completion.
     * @priority                  5
     *
    */
    activate(subscriptionHash: Bytes32, txData?: TxData): Promise<TxHash>;
    /**
     * Get subscription
     *
     * Get a subscription using the subscription hash.
     *
     * @param subscriptionHash  Unique subscription hash returned upon subscribing.
     *
     * @response
     * ```
     * {
     *    owner: '0xdfLqn83...',
     *    tokenAddress: '0xw9s3mfd...',
     *    planHash: '0xwjw93m3sd...',
     *    lastPaymentDate: 155321434, // (epoch in seconds)
     *    terminationDate: 155452222, // (epoch in seconds)
     *    subscriptionHash: '0xhad93d...',
     * }
     * ```
     *
     * @returns                 A subscription object.
     * @priority                6
     *
    */
    get(subscriptionHash: Bytes32): Promise<Subscription>;
    /**
     * Get status
     *
     * Determine the current state of a subscription. It can either be 'active', 'processing' or 'inactive'.
     *
     * The processing state is a simply divisor of the interval of your subscription. Suppose the interval divisor is set at 7, then a 28-day subscription has a maximum of 4 days (plus an hour) to be processed.
     *
     * The extra hour is counted in the case a service node doesn't process a subscription and another node on the network can steal their tokens and process your subscription instead.
     *
     * When you call this function, we'll also let you know whether the user has enough tokens to pay for the subscription and if they've revoked authorisation for 8x to take tokens from their wallet.
     *
     * You can decide how you'd like to handle unprocessed subscriptions.
     *
     * @response
     * ```
     * // In the following response, we see the status of a user who's subscription is being processed but has a low chance of not fulfilling their recurring obligation.
     * [
     *    'processing', // The subscription is being processed
     *    'true', // The user has enough tokens to pay for the subscription
     *    'true', // The user has given 8x enough allowance to take tokens from their wallet
     * ]
     * ```
     *
     * @param subscriptionHash        Unique subscription hash returned upon subscribing.
     * @returns                       Current status, does user has enough tokens, does 8x got authorisation.
     * @priority                      7
     *
     */
    getStatus(subscriptionHash: Bytes32): Promise<[string, boolean, boolean]>;
    /**
     * All subscriptions
     *
     * Get all the subscriptions a user has subscribed to..
     *
     * @param user       The user you'd like to get subscriptions for.
     *
     * @returns          An array of subscription objects.
     * @priority         8
     *
    */
    getSubscribed(user: Address): Promise<Subscription[]>;
    /**
     * Cancel a subscription
     *
     * Cancels a subscription and prevents payments from being taken next billing cycle.
     * This can only be called by the user who created the subscription in the first place.
     *
     * @param subscriptionHash    Unique subscription hash returned upon subscribing.
     * @param txData              Provide signer, gas and gasPrice information (optional).
     *
     * @returns                   Hash of the transaction upon completion.
     * @priority                  9
     *
     */
    cancel(subscriptionHash: Bytes32, txData?: TxData): Promise<TxHash>;
}
